function pbfd_design_filters;

main_data = get(gcf,'UserData');

tag_axes_top    = 'demo_pbfd_axes_top';
tag_axes_bottom = 'demo_pbfd_axes_bottom';

desired_shape = main_data.des_freq_res_lin;
for k=main_data.N_FFT/2+2:main_data.N_FFT
    desired_shape(k)    = desired_shape(main_data.N_FFT+2-k);
end;
desired_shape     = abs(desired_shape).^2;
desired_shape_tmp = desired_shape;

r_FIR                   = ifft(desired_shape_tmp);
r_IIR                   = ifft(1./desired_shape_tmp);

[a_FIR,E_FIR]           = levinson(r_FIR,main_data.N_FIR);
[b_IIR,E_IIR]           = levinson(r_IIR,main_data.N_IIR);

q_FIR                   = E_FIR / (r_FIR(1)+eps);
q_IIR                   = E_IIR / (r_IIR(1)+eps);

b_FIR                   = sqrt(E_FIR);
a_IIR                   = sqrt(E_IIR);

H_FIR                   = freqz(b_FIR,a_FIR,main_data.N_FFT,1,'whole');
H_IIR                   = freqz(b_IIR,a_IIR,main_data.N_FFT,1,'whole');

                     
set(gcf,'CurrentAxes',findobj('Tag',tag_axes_top)); 
if (q_FIR < q_IIR)
   handle_1 = plot((0:main_data.N_FFT/2)/(main_data.N_FFT/2),10*log10(desired_shape(1:end/2+1)+eps),'k',...
                   (0:main_data.N_FFT/2)/(main_data.N_FFT/2),20*log10(abs(H_FIR(1:end/2+1))+eps),'b',...
                   (0:main_data.N_FFT/2)/(main_data.N_FFT/2),20*log10(abs(H_IIR(1:end/2+1))+eps),'r--',...          
                   'LineWidth',2);
else
   handle_1 = plot((0:main_data.N_FFT/2)/(main_data.N_FFT/2),10*log10(desired_shape(1:end/2+1)+eps),'k',...
                   (0:main_data.N_FFT/2)/(main_data.N_FFT/2),20*log10(abs(H_FIR(1:end/2+1))+eps),'b--',...
                   (0:main_data.N_FFT/2)/(main_data.N_FFT/2),20*log10(abs(H_IIR(1:end/2+1))+eps),'r',...          
                   'LineWidth',2);    
end;        
hold on;
handle_2 = plot(main_data.des_freq_supp_p_x,main_data.des_freq_supp_p_y,'bo','LineWidth',2);
hold off;
axis([0 1 -30 30]);
grid on;
xlabel('Normalized frequency / \pi');
ylabel('dB');
if (q_FIR < q_IIR)
   legend('Desired shape','FIR approximation (selected)','IIR approximation');
else
   legend('Desired shape','FIR approximation','IIR approximation (selected)'); 
end;
drawnow;
set(gca,'Tag',tag_axes_top);



set(gcf,'CurrentAxes',findobj('Tag',tag_axes_bottom)); 
if (q_FIR < q_IIR)
   handle_2 = plot((0:main_data.N_FFT/2)/(main_data.N_FFT/2),10*log10(desired_shape(1:end/2+1)+eps)-20*log10(abs(H_FIR(1:end/2+1))+eps),'b',...
                   (0:main_data.N_FFT/2)/(main_data.N_FFT/2),10*log10(desired_shape(1:end/2+1)+eps)-20*log10(abs(H_IIR(1:end/2+1))+eps),'r--',...          
                   'LineWidth',2);
else
   handle_2 = plot((0:main_data.N_FFT/2)/(main_data.N_FFT/2),10*log10(desired_shape(1:end/2+1)+eps)-20*log10(abs(H_FIR(1:end/2+1))+eps),'b--',...
                   (0:main_data.N_FFT/2)/(main_data.N_FFT/2),10*log10(desired_shape(1:end/2+1)+eps)-20*log10(abs(H_IIR(1:end/2+1))+eps),'r',...          
                   'LineWidth',2);    
end;
err_log_FIR = mean(abs(10*log10(desired_shape(1:end/2+1)+eps)-20*log10(abs(H_FIR(1:end/2+1))+eps)));
err_log_IIR = mean(abs(10*log10(desired_shape(1:end/2+1)+eps)-20*log10(abs(H_IIR(1:end/2+1))+eps)));
axis([0 1 -30 30]);
grid on;
xlabel('Normalized frequency / \pi');
ylabel('dB');
if (q_FIR < q_IIR)
    legend(['FIR approximation (selected), average log. error: ',num2str(err_log_FIR,4),' dB'],['IIR approximation, average log. error: ',num2str(err_log_IIR,4),' dB'])
else
    legend(['FIR approximation, average log. error: ',num2str(err_log_FIR,4),' dB'],['IIR approximation (selected), average log. error: ',num2str(err_log_IIR,4),' dB'])
end;
drawnow;
set(gca,'Tag',tag_axes_bottom);

